package com.altinity.opsview.odw.elements;

/**
 * Copyright (C) 2008 Opsera Limited. All rights reserved
 *
 * This file is part of Opsview
 * 
 * Opsview is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * Opsview is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Opsview; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 * 
 */

import java.util.Calendar;
import java.util.List;
import java.util.Map;

import org.dom4j.Element;
import org.jfree.chart.JFreeChart;

import com.altinity.opsview.jfreechart.ThresholdWeekChart;
import com.altinity.opsview.odw.ReportConstants;
import com.altinity.opsview.odw.bc.ODWReportLogger;
import com.altinity.opsview.odw.dao.ODWReportConfigDAO;
import com.altinity.opsview.odw.dao.ODWReportDAO;
import com.altinity.opsview.odw.exception.ReportException;
import com.altinity.opsview.odw.model.HostGroup;
import com.altinity.opsview.odw.reports.ReportCriteria;
import com.excira.jfreechart.XYPoint;

public class AverageAvailibilityForHostGroupsChart extends
		AbstractReportElement implements ReportConstants {
	
	private Map <HostGroup, List <XYPoint>> pointsByHG;

	/**
	 * 
	 */
	public AverageAvailibilityForHostGroupsChart(ReportCriteria rc, ODWReportDAO dao, ODWReportConfigDAO configDao, ODWReportLogger log) {
		super(rc, dao, configDao, log);
	}

	/**
	 * 
	 */
	public Element addElement(Element elements) {
		
		// Create element for graphs
		Element averageAvailabilityGraphs = elements.addElement("element");
		averageAvailabilityGraphs.addAttribute("name", "averageAvailabilityGraphForHostGroup");	
		averageAvailabilityGraphs.addElement("title").setText(
				"Hostgroup Summary");
		
		// Report error if no data exists
		if ( isEmpty() ) {
			
			// Add empty element
			averageAvailabilityGraphs.addElement("empty").setText("true");
			
			// Report error
			log.getErrors().add("No data found for module 'Hostgroup Summary'");
			
			return averageAvailabilityGraphs;
		}
				
		// Otherwise data has been found, create charts
		Element charts = averageAvailabilityGraphs.addElement("charts");
		
		// Iterate over host groups and create graph for each
		String encodedHostGroup = "";
		String fileName = "";
		for ( HostGroup hostGroup : pointsByHG.keySet() ) {
			
			List <XYPoint> points = pointsByHG.get(hostGroup);
			
			// Don't show graph if there are no data points
			// for this host group
			if ( points.isEmpty() )
				continue;
			
			Element chartEl = charts.addElement("chart");
			
			// Chart is plotted over a period of four months
			// Get the correct start date for the date range
			Calendar cal = Calendar.getInstance();
			cal.setTime(rc.getReportStartDate());
			cal.add(Calendar.MONTH, -3);
			
			ThresholdWeekChart weekChart =
				new ThresholdWeekChart(hostGroup.getWarning(), hostGroup.getCritical(), cal.getTime(), rc.getReportEndDate());
			
			JFreeChart chart = 
				weekChart.createChart("", "Availability % for " + hostGroup.getName(), points, true);
			
			encodedHostGroup = "averageAvailability_" + hostGroup.getName().replace(" ", "_");
			fileName = saveImageToTempDir(chart, encodedHostGroup);
			
			chartEl.addElement("title").setText(
					hostGroup.getName() + ": Daily Availability, Last 4 Months ");
			
			chartEl.addElement("filePath").setText(fileName);
		}

		return averageAvailabilityGraphs;
	}
	
	/**
	 * 
	 */
	public void runQuery() throws ReportException {
		
		// Get the average availability chart for each host group
		pointsByHG = 
			dao.getAverageAvailabilityTimeSeriesForHostGroups(configDao, rc);
	}
	
	/**
	 * 
	 */
	private boolean isEmpty() {
		
		// Iterate over XYPoint list for each host group
		// and check that data exists for at least
		// one host group
		for ( HostGroup hg : pointsByHG.keySet() ) {
			
			List <XYPoint> points = pointsByHG.get(hg);
			if ( !points.isEmpty() )
				return false;
		}
		return true;
	}
}
