package com.altinity.opsview.odw.elements;

import java.util.ArrayList;
import java.util.List;

import org.dom4j.Element;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryLabelPositions;

import com.altinity.opsview.odw.ReportConstants;
import com.altinity.opsview.odw.bc.ODWReportLogger;
import com.altinity.opsview.odw.dao.ODWDAO;
import com.altinity.opsview.odw.dao.ODWReportConfigDAO;
import com.altinity.opsview.odw.dao.ODWReportDAO;
import com.altinity.opsview.odw.model.ServiceCheckGroup;
import com.altinity.opsview.odw.model.WeeklyServiceAvailability;
import com.altinity.opsview.odw.reports.ReportCriteria;
import com.excira.jfreechart.BarChart;
import com.excira.jfreechart.BarData;

public class WeeklyServiceSummary extends AbstractReportElement implements ReportConstants {
	
	protected List <ServiceCheckGroup> serviceCheckGroups;
	protected List <BarData> data;
	protected boolean useIndex;
	
	/**
	 *
	 */
	public WeeklyServiceSummary(ReportCriteria rc, ODWReportDAO dao, ODWReportConfigDAO configDao, 
			ODWReportLogger log, List <ServiceCheckGroup> groups) {
		super(rc, dao, configDao, log);
		
		this.serviceCheckGroups = groups;
	}

	/**
	 * 
	 */
	public Element addElement(Element elements) {
		
		Element summaryEl = elements.addElement("element");
		summaryEl.addElement("title").setText("Weekly Availability Summary");
		summaryEl.addAttribute("name", "weeklySummary");
		
		// Handle case where data has been found
		if ( !data.isEmpty() ) {
			
			addChart(summaryEl, data, "weeklySummaryChart", CategoryLabelPositions.DOWN_45);
		}
		// Otherwise no data has been found
		else {
			
			// Add empty element
			summaryEl.addElement("empty").setText("true");
			
			// Report error
			log.getErrors().add("No data found for module 'Weekly Availability Summary'");
		}
		
		return summaryEl;
	}
	
	/**
	 * 
	 */
	public void runQuery() {
		
		// We need to determine if these servicecheck labels
		// are to prepended with an index
		setUseIndex(serviceCheckGroups);
		
		data = new ArrayList<BarData>();
		
		// Now query availability for each servicecheck
		int index = 1;
		for ( ServiceCheckGroup group : serviceCheckGroups ) {
			
			// Add the availability for this servicecheck to the list
			WeeklyServiceAvailability summary = dao.getWeeklyAvailabilityForServiceCheck(rc, group.getIds());
			if ( summary.isEmpty() )
				continue;
			
			String text = group.getText();
			if ( useIndex )
				text = index + ": " + text;
			
			BarData barData = new BarData(summary.getPercentOk(), ODWDAO.PERCENT_OK_INDEX, text);
			data.add(barData);
			
			barData = new BarData(summary.getPercentScheduled(), ODWDAO.PERCENT_SCHED_INDEX, text);
			data.add(barData);
			
			barData = new BarData(summary.getPercentUnscheduled(), ODWDAO.PERCENT_UNSCHED_INDEX, text);
			data.add(barData);
			
			index++;
		}
	}
	
	/**
	 * 
	 */
	private void setUseIndex(List <ServiceCheckGroup> groups) {
		
		int maxLength = 0;
		for ( ServiceCheckGroup group : groups ) {
			
			int length = group.getText().length();
			if ( length > maxLength )
				maxLength = length;

		}
		
		// Chart will only show labels of length less than 28
		// If the label is longer than 28 characters, we want to
		// prepend the label with an index
		useIndex = maxLength >= 28;
	}	
	
	/**
	 * 
	 */
	protected void addChart(Element parentEl, List<BarData> data, String fileNameBase, CategoryLabelPositions position) {
		
		BarChart barChart = new BarChart();
		barChart.createDomainAxis("", position);
		barChart.createRangeAxis(data, "", 25d);
		barChart.setHideLegend(true);
		JFreeChart chart = barChart.createChart("", "", data, false);
		String fileName = saveImageToTempDir(chart, fileNameBase);
		
		parentEl.addElement("filePath").setText(fileName);
	}	
}
