package com.altinity.opsview.odw.reports;

/**
 * Copyright (C) 2008 Opsera Limited. All rights reserved
 *
 * This file is part of Opsview
 * 
 * Opsview is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * Opsview is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Opsview; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 * 
 */


import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jfree.chart.JFreeChart;

import com.altinity.opsview.odw.dao.QueryBuilder;
import com.excira.jfreechart.BarChart;
import com.excira.jfreechart.Report;
import com.excira.jfreechart.WeekChart;


public class SummarySection {
	
	/**
	 * Summary subreport contains:
	 * 1. Overall averages last week (table) 
	 * 2. Average availability last week (time series graph)
	 * 3. Availability trend over last 4 weeks (bar graph)
	 * 4. Availability for week for host group (table)
	 */
	public void generateSection(Report report, String startDate) {
		
		List subReports = report.getSubReports();
		
		Connection connection = null;
		
		// Add overall averages last week
		// Table is included in availabilityReport.jrxml - do
		// not add as subreport
		report.getParameters().putAll(getWeeklyOverallTableReport(connection, startDate));
		
		// Add comments for weekly overall table
		report.getParameters().put("summaryComments", getComments(connection, "summary"));
		
		// Add availability last week
		report.getParameters().put("availabilityTimeSeries", getAvailTimeSeries(connection, startDate));
		
		// Add comments for availability
		report.getParameters().put("availabilityComments", getComments(connection, "availability"));

		// Add availability trend over last four weeks
		report.getParameters().put("availabilityChart", getAvailChart(connection, startDate));
		
		// Add comments for trend
		report.getParameters().put("trendComments", getComments(connection, "trend"));
		
		// Add availability for week for host group
		subReports.add(getWeeklyAvailCrossTabReport());
		
		// Add comments for trend
		report.getParameters().put("downtimeComments", getComments(connection, "downtime"));		
		
		System.out.println("Including: Summary section");
	}
	
	/**
	 * 
	 */
	private Map getWeeklyOverallTableReport(Connection connection, String startDate) {
		
		Map params = new HashMap();
		
		try {
			
			Statement statement = connection.createStatement();
			ResultSet resultSet = statement.executeQuery(QueryBuilder.getAllForWeek(startDate));
			params = ReportUtils.generateParamMap(resultSet);
			
		} catch (Exception e) {
		
			System.out.println("Error in SummaryReport.getAvailChart()");
			e.printStackTrace();
		}
		
		return params;
	}
	
	/**
	 * 
	 */
	private String getComments(Connection connection, String commentsKey) {
		
		StringBuffer comments = new StringBuffer();
		
		try {
			
			Statement statement = connection.createStatement();
			ResultSet resultSet = statement.executeQuery(QueryBuilder.getComments(commentsKey));
			
			// Iterator over result set and append text to comments
			while ( resultSet.next() ) {
				
				comments.append(resultSet.getString("text"));
			}
		} catch (Exception e) {
		
			System.out.println("Error in SummaryReport.getComments() for commentsKey '" + commentsKey + "'");
			e.printStackTrace();
		}
		return comments.toString();
	}
	
	/**
	 * 
	 */
	private Report getWeeklyAvailCrossTabReport() {
		
		return new Report("weeklyAvailCrossTab");
	}
	
	/** 
	 * 
	 */
	private JFreeChart getAvailTimeSeries(Connection connection, String startDate) {
		
		JFreeChart chart = null;
		
		try {
			
			Statement statement = connection.createStatement();
			ResultSet resultSet = statement.executeQuery(QueryBuilder.getPerecentOKForWeek(startDate));
			
			Map indices = new HashMap();
			indices.put(ReportUtils.X_INDEX, new Integer(1));
			indices.put(ReportUtils.Y_INDEX, new Integer(2));
			List data = ReportUtils.generateXYPoints(indices, resultSet);
			
			chart = new WeekChart().createChart("", "Availability", data, true);
			
		} catch (Exception e) { 
		
			System.out.println("Error in SummaryReport.getAvailChart()");
			e.printStackTrace();
		}
		return chart;
	}
	
	/**
	 * 
	 */
	private JFreeChart getAvailChart(Connection connection, String startDate) {
		
		JFreeChart chart =null;
		
		try {
			
			Statement statement = connection.createStatement();
			ResultSet resultSet = statement.executeQuery(QueryBuilder.getAllForMonth(startDate));
			
			List data = ReportUtils.generateBarData(resultSet);
			
			chart = new BarChart().createChart("", "Availability %", data, false);
			
		} catch (Exception e) {
		
			System.out.println("Error in SummaryReport.getAvailChart()");
			e.printStackTrace();
		}
		return chart;
	}
}
