package com.excira.jfreechart;

/**
 * Copyright (C) 2008 Opsera Limited. All rights reserved
 *
 * This file is part of Opsview
 * 
 * Opsview is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * Opsview is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Opsview; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 * 
 */

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Iterator;
import java.util.List;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.LegendItemSource;
import org.jfree.chart.annotations.XYAnnotation;
import org.jfree.chart.annotations.XYDrawableAnnotation;
import org.jfree.chart.annotations.XYPointerAnnotation;
import org.jfree.chart.axis.Axis;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.DateTickMarkPosition;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.data.general.Dataset;
import org.jfree.data.time.Day;
import org.jfree.data.time.RegularTimePeriod;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.IntervalXYDataset;
import org.jfree.ui.RectangleInsets;
import org.jfree.ui.TextAnchor;

/**
 * @author Administrator
 *
 */
public abstract class TimeSeriesChart implements Chart {
	
	/**
	 * 
	 */
	public JFreeChart createChart(String chartTitle, String seriesName, List xyPoints, boolean allowUpdates) {
		
		// Generate visual representation of single (x,y)
		XYItemRenderer renderer = (XYItemRenderer)getRenderer();
		
		// Setup date (x) axis
		DateAxis dateAxis = (DateAxis)getDomainAxis();
		
		// Setup value (y) axis
		ValueAxis  valueAxis = (ValueAxis)getRangeAxis(xyPoints,seriesName);
		
		// Now create the actual plot from the specified list of xy points
		IntervalXYDataset dataset = (IntervalXYDataset)getDataSet(seriesName, xyPoints, allowUpdates);
		XYPlot plot = new XYPlot(dataset, dateAxis, valueAxis, renderer);
		plot.setBackgroundPaint(Color.WHITE);
	
		setRangeAxis((NumberAxis)plot.getRangeAxis(), xyPoints);
		setDomainAxis((DateAxis)plot.getDomainAxis(), xyPoints);
		
		// Add annotations
		addAnnotations(plot, xyPoints);
		
		// Add range markers
		addRangeMarker(plot);
		
		// Finally! Create that chart!
		JFreeChart chart = 
			new JFreeChart(chartTitle, JFreeChart.DEFAULT_TITLE_FONT, plot, true);
	
		chart.setBorderVisible(true);
		chart.setPadding(new RectangleInsets(20,20,20,20));
		
		updateLegend(chart);

		return chart;
	}
	
	/**
	 * 
	 */
	protected void updateLegend(JFreeChart chart) {
		
		chart.removeLegend();
	}
	
	/**
	 * 
	 */
	public LegendItemSource getRenderer() {
		
		XYItemRenderer renderer = new StandardXYItemRenderer();

        renderer.setSeriesPaint(0,Color.BLACK);

		renderer.setBaseToolTipGenerator(new StandardXYToolTipGenerator(
				StandardXYToolTipGenerator.DEFAULT_TOOL_TIP_FORMAT, new SimpleDateFormat("d-MMM-yyyy"),
				new DecimalFormat("0.00")));
		
		return renderer;
	}
	
	/**
	 * 
	 */
	public Axis getDomainAxis() {
		
		DateAxis dateAxis = new DateAxis("Date");
		dateAxis.setTickMarkPosition(DateTickMarkPosition.MIDDLE);
		return dateAxis;
	}
	
	/**
	 * 
	 */
	public Axis getRangeAxis(List xyPoints, String label) {
		
		return new NumberAxis(label);
	}
	
	/**
	 * 
	 */
	protected void setRangeAxis(NumberAxis axis, List xyPoints) {
		
		axis.setAutoRange(false);
		
		if ( xyPoints.isEmpty() ) {
			axis.setRange(0, 100);
			return;
		}		
		
		// Get the min and max for range
		// Set initial min and max to be value
		// of first xyPoint
		double min = ((XYPoint)xyPoints.get(0)).getY();
		double max = ((XYPoint)xyPoints.get(0)).getY();
		
		Iterator i = xyPoints.iterator();
		while ( i.hasNext() ) {
			
			XYPoint xyPoint = (XYPoint)i.next();
			double y = xyPoint.getY();

			if ( min > y )
				min = y;
			
			if ( max < y )
				max = y;	
		}
		
		axis.setRange(min, max);
	}
	
	/**
	 * 
	 */
	protected void setDomainAxis(DateAxis axis, List xyPoints) {
		
	}
	
	/**
	 * 
	 */
	protected void setRangeAxis(NumberAxis axis, double min, double max) {
		
		axis.setAutoRange(false);
		axis.setRange(min, max);
	}
	
	/**
	 * 
	 */
	public Dataset getDataSet(String seriesName, List xyPoints, boolean allowUpdates) {
		
		TimeSeries series = getTimeSeries(seriesName);
		
		Iterator i = xyPoints.iterator();
		while ( i.hasNext() ) {
			
			XYPoint point = (XYPoint)i.next();
			
			// Get the x (date) value and convert to RegularTimePeriod
			RegularTimePeriod x = (Day)getX(point);
			
			// Get the y value
			double y = point.getY();
			
			// Add point to series
			if ( allowUpdates )
				series.addOrUpdate(x, y);
			else
				series.add(x, y);
		}
		
		return new TimeSeriesCollection(series);
	}
	
	/**
	 * 
	 */
	private void addAnnotations(XYPlot plot, List xyPoints) {
		
		Iterator points = xyPoints.iterator();
		while ( points.hasNext() ) {
			
			// Check if this point has annotation
			XYPoint point = (XYPoint)points.next();
			String text = point.getAnnotation();
			//if ( StringUtils.notNullOrEmpty(text) ) {
			if ( text != null & !text.equals("") ) {
				// Create circle!
				CircleDrawer cd = new CircleDrawer(Color.red, new BasicStroke(1.0f), null);
				
				// Get x point
				RegularTimePeriod day = (Day)getX(point);
				double millis = day.getFirstMillisecond();
				
				// Get y point
				double y = point.getY();
				
				// Add annotation to plot
				XYAnnotation annotation = new XYDrawableAnnotation(millis, y, 1, 1, cd);
				plot.addAnnotation(annotation);
				
				// Add pointer
				XYPointerAnnotation pointer = 
					new XYPointerAnnotation(text, millis, y, 45); //3.0 * Math.PI / 4.0
				pointer.setBaseRadius(35.0); // distance between point and tip
				pointer.setTipRadius(3.0);
				pointer.setFont(new Font("SansSerif", Font.PLAIN, 9));
				pointer.setPaint(Color.blue);
				pointer.setTextAnchor(TextAnchor.HALF_ASCENT_RIGHT);
				plot.addAnnotation(pointer);
			}			
		}
	}
	
	/**
	 * 
	 */
	protected void addRangeMarker(XYPlot plot) {
		
	}
	
	/**
	 * Override this method if chart time series is month,yeah etc
	 */
	protected abstract TimeSeries getTimeSeries(String seriesName);
	
	/**
	 * Override this method if chart time series is month, year etc
	 */
	protected abstract Object getX(XYPoint point);
}
